
package ;

import flash.display.DisplayObject;
import flash.display.Graphics;
import flash.display.Sprite;
import flash.events.Event;
import flash.Lib;
import com.redx.geom.Point;
import com.redx.utils.IntIterPlus;

class Grid extends Sprite {

	public var w:Int;
	public var h:Int;
	public var tileW:Int;
	public var tileH:Int;
	public var tileMid:Point;

	private var spr:Sprite;
	private var pointsGr:Array<Sprite>;
	private var iter:IntIterPlus;

	public function new(width:Int, height:Int, tW:Int, tH:Int) {
		super();

		pointsGr = new Array<Sprite>();
		iter = new IntIterPlus(0, 1);

		w = width;
		h = height;
		tileW = tW;
		tileH = tH;
		tileMid = new Point(cast(tileW / 2, Int), cast(tileH / 2, Int));
		
		spr = new Sprite();
		addChild(spr);

	}

	public function draw(thickness:Float = 1, color:UInt = 0) {
		var i:Int;
		var g:Graphics = spr.graphics;
		g.lineStyle(thickness, color);

		iter.min = 0;		
		iter.max = w;
		for (i in iter) {
			g.moveTo(i * tileW, 0);
			g.lineTo(i * tileW, h * tileH);
		}

		iter.min = 0;
		iter.max = h;
		for (i in iter) {
			g.moveTo(0, i * tileH);
			g.lineTo(w * tileW, i * tileH);
		}
	}

	public function clear() {
		spr.graphics.clear();
	}
	
	public function drawPoint(gridx:Int, gridy:Int, radius:Int = 1, color:UInt = 0):Sprite {
		var realX:Int = cast(gridx * tileW + tileW / 2, Int);
		var realY:Int = cast(gridy * tileH + tileH / 2, Int);
		
		var pt:Sprite = new Sprite();
		pt.x = realX;
		pt.y = realY;
		pt.graphics.lineStyle(2, color);
		if (radius == 0) pt.graphics.drawCircle(0, 0, tileW / 2 - 2);
		else pt.graphics.drawCircle(0, 0, radius);
		addChild(pt);
		pointsGr.push(pt);
		
		return pt;
	}
	
	public function fillTile(gridx:Int, gridy:Int, color:UInt = 0, alpha:Float = 1) {
		var realX:Int = gridx * tileW;
		var realY:Int = gridy * tileH;
		
		var fill:Sprite = new Sprite();
		fill.x = realX;
		fill.y = realY;
		fill.graphics.beginFill(color, alpha);
		fill.graphics.drawRect(0, 0, tileW, tileH);
		fill.graphics.endFill();
		
		addChild(fill);
	}

	public function removeFill(gridx:Int, gridy:Int) {
		var realX:Int = gridx * tileW + 2;
		var realY:Int = gridy * tileH + 2;

		var objects:Array<DisplayObject> = this.getObjectsUnderPoint(new flash.geom.Point(realX, realY));
		if (objects.length == 0) return;
		var fill:DisplayObject = objects[0];
		removeChild(fill);
	}
	
	public inline function globalToGrid(x:Int, y:Int):Point {
		return new Point(Math.floor(x / tileW), Math.floor(y / tileH));
	}

	public inline function globalPointToGrid(pt:Point):Point {
		//pt.x = cast(pt.x / tileW);
		//pt.y = cast(pt.y / tileH);
		//return pt;
		return new Point(cast(pt.x / tileW), cast(pt.y / tileH));
	}
	
	public inline function gridMidToGlobalX(x:Int):Int {
		return cast((x * tileW) + (tileW/2), Int);
	}
	
	public inline function gridMidToGlobalY(y:Int):Int {
		return cast((y * tileH) + (tileH/2), Int);
	}
	
	public inline function clearPts() {
		//iter.max = pointsGr.length;

		for (i in 0...pointsGr.length) {
			removeChild(pointsGr[i]);
		}
		pointsGr = [];
	}

}